/*
 * SIP Communicator, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package net.java.sip.communicator.plugin.customjabberwizz;

import java.util.*;

import net.java.sip.communicator.service.browserlauncher.*;
import net.java.sip.communicator.service.gui.*;
import net.java.sip.communicator.service.ldap.*;
import net.java.sip.communicator.service.protocol.*;
import net.java.sip.communicator.service.resources.*;
import net.java.sip.communicator.util.*;

import org.jitsi.service.configuration.*;
import org.jitsi.service.resources.*;

import org.osgi.framework.*;

/**
 * Registers the <tt>GoogleTalkAccountRegistrationWizard</tt> in the UI Service.
 *
 * @author Yana Stamcheva
 */
public class CustomJabberAccRegWizzActivator
    extends AbstractServiceDependentActivator
{
    public static BundleContext bundleContext;

    private static ResourceManagementService resourcesService;

    /**
     * LDAP service.
     */
    private static LdapService ldapService = null;

    private static ConfigurationService configService;

    /**
     * The <tt>Logger</tt> used by the <tt>GoogleTalkAccRegWizzActivator</tt>
     * and its instances for logging output.
     */
    private static final Logger logger
        = Logger.getLogger(CustomJabberAccRegWizzActivator.class);

    private static BrowserLauncherService browserLauncherService;

    private static UIService uiService;

    /**
     * Starts this bundle when ui service is available.
     */
    public void start(Object dependentService)
    {
        uiService = (UIService) dependentService;

        CustomJabberAccountRegistrationWizard wizard =
            new CustomJabberAccountRegistrationWizard(uiService
                .getAccountRegWizardContainer());

        Hashtable<String, String> containerFilter
            = new Hashtable<String, String>();
        containerFilter.put(
                ProtocolProviderFactory.PROTOCOL,
                CustomJabberAccountRegistrationWizard.PROTOCOL);

        bundleContext.registerService(
            AccountRegistrationWizard.class.getName(),
            wizard,
            containerFilter);
    }

    /**
     * The UIService class which this activator is interested in.
     * @return the UIService class name.
     */
    public Class getDependentServiceClass()
    {
        return UIService.class;
    }

    /**
     * Setting context to the activator, as soon as we have one.
     *
     * @param context the context to set.
     */
    public void setBundleContext(BundleContext context)
    {
        bundleContext = context;
    }

    public void stop(BundleContext bundleContext)
        throws Exception
    {
    }

    /**
     * Indicates if a protocol provider for the current protocol is available.
     * @return <tt>true</tt> if there's a registered protocol provider for the
     * current custom protocol, otherwise returns <tt>false</tt>
     */
    public static boolean isCustomJabberAccountAvailable()
    {
        ServiceReference[] serRefs = null;

        String osgiFilter = "("
            + ProtocolProviderFactory.PROTOCOL
            + "=" + ProtocolNames.JABBER + ")";

        try
        {
            serRefs = bundleContext.getServiceReferences(
                ProtocolProviderFactory.class.getName(), osgiFilter);
        }
        catch (InvalidSyntaxException ex)
        {
            logger.error("JabberAccRegWizzActivator : " + ex);
        }

        ProtocolProviderFactory factory
            = (ProtocolProviderFactory) bundleContext.getService(serRefs[0]);

        Iterator<AccountID> accountsIter
            = factory.getRegisteredAccounts().iterator();

        while (accountsIter.hasNext())
        {
            AccountID account = accountsIter.next();

            if (account.getProtocolName().equals(
                CustomJabberAccountRegistrationWizard.PROTOCOL))
            {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns the <tt>ProtocolProviderFactory</tt> for the Google Talk
     * protocol.
     * 
     * @return the <tt>ProtocolProviderFactory</tt> for the Google Talk
     *         protocol
     */
    public static ProtocolProviderFactory getGoogleTalkProtocolProviderFactory()
    {
        ServiceReference[] serRefs = null;

        String osgiFilter = "("
            + ProtocolProviderFactory.PROTOCOL
            + "=" + ProtocolNames.JABBER + ")";

        try
        {
            serRefs = bundleContext.getServiceReferences(
                ProtocolProviderFactory.class.getName(), osgiFilter);
        }
        catch (InvalidSyntaxException ex)
        {
            logger.error("GoogleTalkAccRegWizzActivator : " + ex);
        }

        return (ProtocolProviderFactory) bundleContext.getService(serRefs[0]);
    }

    /**
     * Returns the <tt>UIService</tt>.
     * 
     * @return the <tt>UIService</tt>
     */
    public static UIService getUIService()
    {
        return uiService;
    }

    /**
     * Returns the <tt>BrowserLauncherService</tt> obtained from the bundle
     * context.
     * @return the <tt>BrowserLauncherService</tt> obtained from the bundle
     * context
     */
    public static BrowserLauncherService getBrowserLauncher()
    {
        if (browserLauncherService == null)
        {
            browserLauncherService =
                (BrowserLauncherService) bundleContext
                    .getService(bundleContext
                        .getServiceReference(BrowserLauncherService.class
                            .getName()));
        }

        return browserLauncherService;
    }

    public static ResourceManagementService getResources()
    {
        if (resourcesService == null)
            resourcesService =
                ResourceManagementServiceUtils
                    .getService(CustomJabberAccRegWizzActivator.bundleContext);
        return resourcesService;
    }

    /**
     * Get LDAP service.
     *
     * @return LDAP service
     */
    public static LdapService getLdapService()
    {
        if(ldapService == null)
        {
            ldapService = ServiceUtils.getService(bundleContext,
                    LdapService.class);
        }

        return ldapService;
    }

    /**
     * Returns the service giving access to configuration properties.
     * @return the service giving access to configuration properties
     */
    public static ConfigurationService getConfigService()
    {
        if (configService == null)
            configService = (ConfigurationService) bundleContext.getService(
                bundleContext.getServiceReference(
                    ConfigurationService.class.getName()));

        return configService;
    }
}
